@@ -7,9 +7,11 @@
 #' @param lambda A real \eqn{[0,1]} where 1 means just perfect correlations will be used and 0 no regularization is used.
 #' @param R A strictly positive integer detailing the number of components to build in the model.
 #' @param L0 An integer non nul parameter giving the largest number of X variables that can be selected.
+#' @param mu A real positive. The Ridge parameter changing the bias of the regression model. If is NULL, consider the classical ddsPLS. Default to NULL.
+#' @param deflat Logical. If TRUE, the solution uses deflations to construct the weights.
+#' @param weight Logical. If TRUE, the scores are divided by the number of selected variables in the corresponding block.
 #' @param mode A character chain. Possibilities are "\strong{(reg,lda,logit)}", which implies regression problem, linear discriminant analysis (through the paclkage \code{MASS}, function \code{lda}) and logistic regression (function \code{glm}). Default is \strong{reg}.
 #' @param id_na A list of na indices for each block. Initialized to NULL.
-#' @param verbose Logical. If TRUE, the function cats specificities about the model. Default is FALSE.
 #' @param NZV Float. The floatting value above which the weights are set to 0.
 #'
 #' @return A list containing the following objects:
@@ -45,7 +47,9 @@
 #' @importFrom MASS lda
 #'
 MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
-                         L0=NULL,verbose=FALSE,id_na=NULL,
+                         L0=NULL,mu=NULL,deflat=FALSE,
+                         weight=FALSE,
+                         id_na=NULL,
                          NZV=1e-9){
 
   my_scale <- function(a){
@@ -127,20 +131,19 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
         }
       }
       all_maxs[ii+1:(ps[k])] <- apply(c_k,2,max)
-      # if(is.null(id_na)){
-      #   all_maxs[ii+1:(ps[k])] <- apply(abs(crossprod(Y,Xs[[k]])/(n-1)),MARGIN = 2,max)
-      # }else{
-      #   if(length(id_na[[k]])>0){
-      #     all_maxs[ii+1:(ps[k])] <- apply(abs(crossprod(Y[-id_na[[k]],],Xs[[k]][-id_na[[k]],])/(n-1)),MARGIN = 2,max)
-      #   }else{
-      #     all_maxs[ii+1:(ps[k])] <- apply(abs(crossprod(Y,Xs[[k]])/(n-1)),MARGIN = 2,max)
-      #   }
-      # }
     }
     lambda_L0 <- sort(all_maxs,decreasing = T)[min(sum_ps,1+L0)]
     lambda_in <- rep(lambda_L0,K)
   }
-  Ms <- lapply(1:K,function(k,Xs,Y,l,n){
+  getMS_deflat <- function(X_def,Y_def,lambda_in,k){
+    M0 <- suppressWarnings(cor(Y_def,X_def))
+    M0[which(is.na(M0))] <- 0
+    M <- abs(M0) - lambda_in[k]
+    M[which(M<NZV)] <- 0 ### VERY IMPORTANT
+    M <- sign(M0)*M
+    M
+  }
+  getMS <- function(k,Xs,Y,l,n){
     if(length(id_na[[k]])>0){
       M0 <- suppressWarnings(cor(Y[-id_na[[k]],],Xs[[k]][-id_na[[k]],]))
     }else{
@@ -148,16 +151,18 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
     }
     M0[which(is.na(M0))] <- 0
     M <- abs(M0) - l[k]
-    M[which(M<0)] <- 0
+    M[which(M<NZV)] <- 0 ### VERY IMPORTANT
     M <- sign(M0)*M
     M
-  },Xs,Y,lambda_in,n)
-  if(verbose){
-    N_max <- sum(unlist(lapply(Ms,function(m){length(which(colSums(abs(m))>NZV))})))
-    cat(paste("At most ",N_max," variable(s) can be selected in the X part",sep=""));cat("\n")
   }
+  Ms <- lapply(1:K,getMS,Xs,Y,lambda_in,n)
   ## Solve optimization problem
-  tete <- min(R,q)#min(R,min(unlist(lapply(Ms,dim))))
+  if(deflat){
+    R_max <- n
+  }else{
+    R_max <- q
+  }
+  tete <- min(R,R_max)
   if(R<1){
     stop("Choose R superior to 1",
          call. = FALSE)
@@ -175,7 +180,6 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
   #### Inside problems
   u_t_r = u_t_r_0 <- list()
   t_r <- list()
-  z_r <- list()
   z_t=t_t <- list()
   # BETA_r <- list()
   for(k in 1:K){
@@ -184,29 +188,53 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
                     d=rep(0,R))
     }
     else{
-      # R_k <- min(R,min(dim(Ms[[k]])))
-      # svd_k <- svd(Ms[[k]],nu = 0,nv = R)
-      R_init <- min(q,sum(ps))
-      svd_k_init <- svd(Ms[[k]],nu = 0,nv = R_init)
-      eigen_YXk <- apply(mmultC(Ms[[k]],svd_k_init$v),2,function(t)sum(t^2))
-      eigen_YXk[which(svd_k_init$d<NZV)] <- 0
-      ordo_YXk <- order(eigen_YXk,decreasing = T)[1:min(R,length(eigen_YXk))]
-      svd_k <- list(v=svd_k_init$v[,ordo_YXk,drop=F],
-                    d=svd_k_init$d[ordo_YXk])
-      ## Complete coefficients if needed
-      length_val_prop <- length(svd_k$d)
-      if(length_val_prop<R){
-        svd_k$d <- c(svd_k$d,rep(0,R-length_val_prop))
-      }
-      ## Complete basis if needed
-      ncol_V <- ncol(svd_k$v)
-      if(ncol_V<R){
-        svd_k$v <- cbind(svd_k$v,matrix(0,nrow(svd_k$v),R-ncol_V))
-      }
-      ## Test coefficients and put corresponding vectors to 0 if needed
-      for(r in 1:R){
-        if(svd_k$d[r]==0){
-          svd_k$v[,r] <- 0
+      if(!deflat & is.null(mu)){
+        R_init <- min(q,sum(ps))
+        svd_k_init <- svd(Ms[[k]],nu = 0,nv = R_init)
+        eigen_YXk <- apply(mmultC(Ms[[k]],svd_k_init$v),2,function(t)sum(t^2))
+        eigen_YXk[which(svd_k_init$d<NZV)] <- 0
+        ordo_YXk <- order(eigen_YXk,decreasing = T)[1:min(R,length(eigen_YXk))]
+        svd_k <- list(v=svd_k_init$v[,ordo_YXk,drop=F],
+                      d=svd_k_init$d[ordo_YXk])
+        ## Complete coefficients if needed
+        length_val_prop <- length(svd_k$d)
+        if(length_val_prop<R){
+          svd_k$d <- c(svd_k$d,rep(0,R-length_val_prop))
+        }
+        ## Complete basis if needed
+        ncol_V <- ncol(svd_k$v)
+        if(ncol_V<R){
+          svd_k$v <- cbind(svd_k$v,matrix(0,nrow(svd_k$v),R-ncol_V))
+        }
+        ## Test coefficients and put corresponding vectors to 0 if needed
+        for(r in 1:R){
+          if(svd_k$d[r]==0){
+            svd_k$v[,r] <- 0
+          }
+        }
+      }else{
+        svd_k <- list(v=matrix(0,nrow = ncol(Ms[[k]]),ncol = R),
+                      d=rep(0,R))
+        for(r in 1:R){
+          if(r==1){
+            X_0 <- Xs[[k]]
+            Y_0 <- Y
+          }
+          ## Solve optimisation problem
+          u_r_def <- svd(Ms[[k]],nu = 0,nv = 1)$v
+          svd_k$v[,r] <- u_r_def
+          t_r_def <- mmultC(X_0,u_r_def)
+          ## Perform deflation
+          norm_sc <- sum(t_r_def^2)
+          svd_k$d[r] <- sqrt(norm_sc)
+          if(norm_sc!=0){
+            X_0 <- X_0 - mmultC(t_r_def,crossprod(t_r_def,X_0))/norm_sc
+            if(mode=="reg"){
+              Y_0 <- Y_0 - mmultC(t_r_def,crossprod(t_r_def,Y_0))/norm_sc
+            }
+          }
+          ## Perform soft-thresholding
+          Ms[[k]] <- getMS_deflat(X_0,Y_0,lambda_in,k)
         }
       }
     }
@@ -214,116 +242,139 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
     if(k==1){
       for(r in 1:R){
         t_r[[r]] <- matrix(0,n,K)
-        z_r[[r]] <- matrix(0,q,K)
       }
     }
     for(r in 1:R){
       if(svd_k$d[r]!=0){
         t_r[[r]][,k] <- mmultC(Xs[[k]],u_t_r[[k]][,r,drop=F])
-        z_r[[r]][,k] <- mmultC(Ms[[k]],u_t_r[[k]][,r,drop=F])
       }
     }
     z_t[[k]] <- mmultC(Ms[[k]],u_t_r[[k]])
+    if(weight & svd_k$d[1]!=0){
+      z_t[[k]] <- z_t[[k]]/length(which(rowSums(abs(u_t_r[[k]]))>NZV))
+    }
     t_t[[k]] <- mmultC(Xs[[k]],u_t_r[[k]])#crossprod(Y,Xs[[k]]%*%u_t_r[[k]])
   }
-  ## Big SVD solution ######################### -----------------
-  U_t_super <- list()
-  Z <- do.call(cbind,z_t)
-  R_opt <- R#min(min(dim(Z)),R)
-  svd_Z <- svd(Z,nu = R_opt,nv = R_opt)
-  # ## Reorder well ## -----
-  # crosY_T <- crossprod(Y,do.call(cbind,t_t))
-  # power <- colSums((crosY_T%*%svd_Z$v)^2)
-  # orderGood <- order(power,decreasing = T)
-  # svd_Z$d <- (svd_Z$d[orderGood])[1:R]
-  # svd_Z$u <- (svd_Z$u[,orderGood,drop=F])[,1:R,drop=F]
-  # svd_Z$v <- (svd_Z$v[,orderGood,drop=F])[,1:R,drop=F]
-  ## END --- Reorder well ## -----
-  beta_all <- svd_Z$v
-  beta_list <- list()
-  for(k in 1:K){#r in 1:R){
-    #beta_list[[r]] <- beta_all[K*(r-1)+1:K,,drop=F]
-    beta_list[[k]] <- beta_all[R*(k-1)+1:R,,drop=F]
-  }
-  V_super <- svd_Z$u
-  if(length(svd_Z$d)<R){
-    svd_Z$d <- c(svd_Z$d,rep(0,R-length(svd_Z$d)))
-    V_super <- cbind(V_super,matrix(0,nrow=nrow(V_super),ncol=R-length(svd_Z$d)))
-  }
-  R_here <- ncol(beta_list[[1]])
-  T_super <- matrix(0,nrow=n,ncol=R_here)
-  for(k in 1:K){
-    U_t_super[[k]] <- mmultC(u_t_r[[k]],beta_list[[k]])
-    T_super <- T_super + mmultC(Xs[[k]],U_t_super[[k]])
-  }
-  ###########################################################
-  vars_current <- rep(0,R_here)
-  for(r in 1:R_here){
-    sc_r <- T_super[,r,drop=F]#scale(x$mod$t[,r,drop=F],scale=F)
-    var_t_super_r <- sum(sc_r^2)
-    if(var_t_super_r!=0){
-      if(n>q){
-        deno_left <- norm(crossprod(Y),'f')
+  U_t_super = beta_list <- list()
+  if(is.null(mu)){
+    ## Big SVD solution ######################### -----------------
+    Z <- do.call(cbind,z_t)
+    R_opt <- R
+    svd_Z <- svd(Z,nu = R_opt,nv = R_opt)
+    beta_all <- svd_Z$v
+    for(k in 1:K){
+      beta_list[[k]] <- beta_all[R*(k-1)+1:R,,drop=F]
+    }
+    V_super <- svd_Z$u
+    if(length(svd_Z$d)<R){
+      svd_Z$d <- c(svd_Z$d,rep(0,R-length(svd_Z$d)))
+      V_super <- cbind(V_super,matrix(0,nrow=nrow(V_super),ncol=R-length(svd_Z$d)))
+    }
+    R_here <- ncol(beta_list[[1]])
+    T_super <- matrix(0,nrow=n,ncol=R_here)
+    for(k in 1:K){
+      U_t_super[[k]] <- mmultC(u_t_r[[k]],beta_list[[k]])
+      T_super <- T_super + mmultC(Xs[[k]],U_t_super[[k]])
+    }
+    ###########################################################
+    vars_current <- rep(0,R_here)
+    for(r in 1:R_here){
+      sc_r <- T_super[,r,drop=F]
+      var_t_super_r <- sum(sc_r^2)
+      if(var_t_super_r!=0){
+        if(n>q){
+          deno_left <- norm(crossprod(Y),'f')
+        }else{
+          deno_left <- norm(tcrossprod(Y),'f')
+        }
+        deno <- deno_left*sum(sc_r^2)
+        numer <- sum(mmultC(Y,crossprod(Y,sc_r))*sc_r)
+        vars_current[r] <- numer/deno
+      }
+    }
+    l_cur <- length(vars_current)
+    if(l_cur>1){
+      ord <- order(vars_current,decreasing = T)
+      T_super <- T_super[,ord[1:R],drop=F]
+      V_super <- V_super[,ord[1:R],drop=F]
+      for(k in 1:K){
+        U_t_super[[k]] <- U_t_super[[k]][,ord[1:R],drop=F]
+        beta_list[[k]] <- beta_list[[k]][,ord[1:R],drop=F]
+      }
+    }
+    ###########################################################
+    ## -------------------------- ######################### -----------------
+    S_super <- mmultC(Y,V_super)
+    T_S <- crossprod(T_super,S_super)
+    T_T <- crossprod(T_super)
+    svd_ort_T_super <- svd(T_super,nu = 0,nv = R)
+    v_ort <- svd_ort_T_super$v
+    Delta_ort <- svd_ort_T_super$d^2
+    if(sum(Delta_ort)!=0){
+      t_ort <- mmultC(T_super,v_ort)
+      s_ort <- mmultC(S_super,v_ort)
+      D_0_inv <- matrix(0,nrow = length(Delta_ort),ncol = length(Delta_ort))
+      del_0 <- which(Delta_ort<NZV)
+      if(length(del_0)>0){
+        diag(D_0_inv)[-del_0] <- 1/Delta_ort[-del_0]
       }else{
-        deno_left <- norm(tcrossprod(Y),'f')
+        diag(D_0_inv) <- 1/Delta_ort
       }
-      deno <- deno_left*sum(sc_r^2)
-      numer <- sum(mmultC(Y,crossprod(Y,sc_r))*sc_r)
-      vars_current[r] <- numer/deno#sum(diag(mmultC(tcrossprod(Y),tcrossprod(sc_r))))/deno
-    }
-  }
-  l_cur <- length(vars_current)
-  if(l_cur>1){
-    ord <- order(vars_current,decreasing = T)
-    T_super <- T_super[,ord[1:R],drop=F]
-    V_super <- V_super[,ord[1:R],drop=F]
-    for(k in 1:K){
-      U_t_super[[k]] <- U_t_super[[k]][,ord[1:R],drop=F]
-      # u_t_r[[k]] <- u_t_r[[k]][,ord,drop=F]
-      beta_list[[k]] <- beta_list[[k]][,ord[1:R],drop=F]
-    }
-  }
-  ###########################################################
-  ## -------------------------- ######################### -----------------
-  S_super <- mmultC(Y,V_super)
-  T_S <- crossprod(T_super,S_super)
-  T_T <- crossprod(T_super)
-  # svd_ort <- svd(S_T,nu = R,nv = R)
-  svd_ort_T_super <- svd(T_super,nu = 0,nv = R)
-  # u_ort <- svd_ort_T_super$u
-  v_ort <- svd_ort_T_super$v
-  Delta_ort <- svd_ort_T_super$d^2
-  if(sum(Delta_ort)!=0){
-    t_ort <- mmultC(T_super,v_ort)
-    s_ort <- mmultC(S_super,v_ort)
-    D_0_inv <- matrix(0,nrow = length(Delta_ort),ncol = length(Delta_ort))
-    del_0 <- which(Delta_ort<NZV)
-    if(length(del_0)>0){
-      diag(D_0_inv)[-del_0] <- 1/Delta_ort[-del_0]
+      B_0 <- mmultC(mmultC(v_ort,tcrossprod(D_0_inv,v_ort)),T_S)
     }else{
-      diag(D_0_inv) <- 1/Delta_ort
+      t_ort=s_ort <- matrix(0,nrow = nrow(T_super),ncol=R)
+      B_0 <- matrix(0,nrow = R,ncol=R)
+      V_super <- matrix(0,q,R)
     }
-    B_0 <- mmultC(mmultC(v_ort,tcrossprod(D_0_inv,v_ort)),T_S)
-    # A <- matrix(0,R,R)
-    # for(r in 1:R){
-    #  A[r,r] <- as.numeric(crossprod(t_ort[,r],s_ort[,r]))/as.numeric(crossprod(t_ort[,r]))
-    # }
-  }else{
-    t_ort=s_ort <- matrix(0,nrow = nrow(T_super),ncol=R)
-    B_0 <- matrix(0,nrow = R,ncol=R)
-    # A <- matrix(0,R,R)
-    V_super <- matrix(0,q,R)
-  }
-  u <- beta_all#beta# deprecated
-  if(mode=="reg"){
+    u <- beta_all
+
+  }else{ ## RIDGE solution
     B <- list()
-    count <- 1
+    T_super <- matrix(0,n,q)
+
+    T_super_reg <- matrix(NA,n,R*K)
+    count_reg <- 1
+    for(r in 1:R){
+      for(k in 1:K){
+        T_super_reg[,count_reg] <- t_r[[r]][,k]
+        count_reg <- count_reg + 1
+      }
+    }
+    Q <- mmultC(solve(crossprod(T_super_reg)+n*mu*diag(1,R*K)),
+                crossprod(T_super_reg,Y))
+    count_reg <- 1
     for(k in 1:K){
-      B_k <- tcrossprod(mmultC(U_t_super[[k]],B_0),V_super)
-      if(anyNA(B_k)){
-        B_k <- matrix(0,nrow(B_k),ncol(B_k))
+      B_t <- matrix(NA,R,q)
+      for(r in 1:R){
+        B_t[r,] <- Q[count_reg,]
+        count_reg <- count_reg + 1
+      }
+      B[[k]] <- mmultC(u_t_r[[k]],B_t)
+      for(jj in 1:q){
+        B[[k]][,jj] <- B[[k]][,jj]*sd_y[jj]
+      }
+      T_super <- T_super + mmultC(Xs[[k]],B[[k]])
+    }
+    U_t_super <- B
+    V_super <- diag(1,q)
+    S_super <- Y
+    u <- matrix(rep(diag(1,R),K),R)
+    t_ort=s_ort <- matrix(0,nrow = nrow(T_super),ncol=R)
+  }
+  if(mode=="reg"){
+    if(is.null(mu)){
+      B <- list()
+      count <- 1
+      for(k in 1:K){
+        B_k <- tcrossprod(mmultC(U_t_super[[k]],B_0),V_super)
+        if(anyNA(B_k)){
+          B_k <- matrix(0,nrow(B_k),ncol(B_k))
+        }
+        B[[k]] <- B_k
+        for(jj in 1:q){
+          B[[k]][,jj] <- B[[k]][,jj]*sd_y[jj]
+        }
       }
-      B[[k]] <- B_k
     }
   }else{
     if(is.factor(Y_0)) Y_0 <- as.character(Y_0)
@@ -364,20 +415,10 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
       }
     }
   }
-  if(verbose){
-    a<-lapply(u_t_r,function(u){apply(u,2,function(u){length(which(abs(u)>NZV))})})
-    cat("    For each block of X, are selected in order of component:");cat("\n")
-    for(k in 1:K){
-      cat(paste("        @ (",paste(a[[k]],collapse = ","),") variable(s)",sep=""));cat("\n")
-    }
-    cat("    For the Y block, are selected in order of component:");cat("\n")
-    cat(paste("        @ (",paste(apply(V_super,2,function(u){length(which(abs(u)>NZV))}),
-                                  collapse = ","),") variable(s)",sep=""));cat("\n")
-  }
   list(u=u_t_r,u_t_super=U_t_super,V_super=V_super,ts=t_r,beta_comb=u,
        T_super=T_super,S_super=S_super,
        t_ort=t_ort,s_ort=s_ort,B=B,
-       mu_x_s=mu_x_s,sd_x_s=sd_x_s,mu_y=mu_y,sd_y=sd_y,R=R,q=q,Ms=Ms,lambda=lambda_in[1])
+       mu_x_s=mu_x_s,sd_x_s=sd_x_s,mu_y=mu_y,sd_y=sd_y,R=R,q=q,Ms=Ms,lambda=lambda_in[1],mu=mu)
 }
 
 
@@ -394,12 +435,11 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
 #' @param lambda A real \eqn{[0,1]} where 1 means just perfect correlations will be used and 0 no regularization is used.
 #' @param R A strictly positive integer detailing the number of components to build in the model.
 #' @param L0 An integer non nul parameter giving the largest number of X variables that can be selected.
+#' @param mu A real positive. The Ridge parameter changing the bias of the regression model. If is NULL, consider the classical ddsPLS. Default to NULL.
+#' @param deflat Logical. If TRUE, the solution uses deflations to construct the weights.
+#' @param weight Logical. If TRUE, the scores are divided by the number of selected variables of their corresponding block.
 #' @param keep_imp_mod Logical. Whether or not to keep imputation \strong{mddsPLS} models. Initialized to \code{FALSE} due to the potential size of those models.
-#' @param reg_imp_model Logical. Whether or not to regularize the imputation models. Initialized to \code{TRUE}.
 #' @param mode A character chain. Possibilities are "\strong{(reg,lda,logit)}", which implies regression problem, linear discriminant analysis (through the paclkage \code{MASS}, function \code{lda}) and logistic regression (function \code{glm}). Default is \strong{reg}.
-#' @param errMin_imput Positive real. Minimal error in the Tribe Stage of the Koh-Lanta algorithm. Default is \eqn{1e-9}.
-#' @param maxIter_imput Positive integer. Maximal number of iterations in the Tribe Stage of the Koh-Lanta algorithm. If equals to \eqn{0}, mean imputation is  considered. Default is \eqn{5}.
-#' @param verbose Logical. If TRUE, the convergence progress of the Koh-Lanta algorithm is reported. Default is FALSE.
 #' @param NZV Float. The floatting value above which the weights are set to 0.
 #' @param getVariances Logical. Whether or not to compute variances. Default is \emph{TRUE}.
 #'
@@ -433,14 +473,14 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
 #' X <- penicilliumYES$X
 #' X <- scale(X[,which(apply(X,2,sd)>0)])
 #' Y <- as.factor(unlist(lapply(c("Melanoconidiu","Polonicum","Venetum"),function(tt){rep(tt,12)})))
-#' # mddsPLS_model_class <- mddsPLS(Xs = X,Y = Y,R = 2,L0=3,mode = "lda",verbose = TRUE)
+#' # mddsPLS_model_class <- mddsPLS(Xs = X,Y = Y,R = 2,L0=3,mode = "lda")
 #' # summary(mddsPLS_model_class,plot_present_indiv = FALSE)
 #'
 #' ## Regression example :
 #' data("liverToxicity")
 #' X <- scale(liverToxicity$gene)
 #' Y <- scale(liverToxicity$clinic)
-#' #mddsPLS_model_reg <- mddsPLS(Xs = X,Y = Y,L0=10,R = 1, mode = "reg",verbose = TRUE)
+#' #mddsPLS_model_reg <- mddsPLS(Xs = X,Y = Y,L0=10,R = 1, mode = "reg")
 #' #summary(mddsPLS_model_reg)
 #'
 #' # Multi-block example :
@@ -451,7 +491,7 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
 #' Xs <- list(X[,1:1000],X[,-(1:1000)])
 #' Xs[[1]][1:5,]=Xs[[2]][6:10,] <- NA
 #' Y <- as.factor(unlist(lapply(c("Melanoconidiu","Polonicum","Venetum"),function(tt){rep(tt,12)})))
-#' #mddsPLS_model_class <- mddsPLS(Xs = Xs,Y = Y,L0=3,mode = "lda",R = 2,verbose = TRUE)
+#' #mddsPLS_model_class <- mddsPLS(Xs = Xs,Y = Y,L0=3,mode = "lda",R = 2)
 #' #summary(mddsPLS_model_class)
 #'
 #' ## Regression example :
@@ -460,12 +500,13 @@ MddsPLS_core <- function(Xs,Y,lambda=0,R=1,mode="reg",
 #' Xs <- list(X[,1:1910],X[,-(1:1910)])
 #' Xs[[1]][1:5,]=Xs[[2]][6:10,] <- NA
 #' Y <- scale(liverToxicity$clinic)
-#' #mddsPLS_model_reg <- mddsPLS(Xs = Xs,Y = Y,lambda=0.9,R = 1, mode = "reg",verbose = TRUE)
+#' #mddsPLS_model_reg <- mddsPLS(Xs = Xs,Y = Y,lambda=0.9,R = 1, mode = "reg")
 #' #summary(mddsPLS_model_reg)
-mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
-                    keep_imp_mod=FALSE,reg_imp_model=TRUE,
-                    errMin_imput=1e-9,maxIter_imput=50,
-                    verbose=FALSE,NZV=1E-9,getVariances=TRUE){
+mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",
+                    L0=NULL,mu=NULL,deflat=FALSE,
+                    weight=FALSE,
+                    keep_imp_mod=FALSE,
+                    NZV=1E-9,getVariances=TRUE){
 
   my_scale <- function(a){
     if(!is.matrix(a)){
@@ -536,12 +577,13 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
       }
     }
     R <- length(x$mod$ts)
+    R_super <- ncol(x$mod$T_super)
     VAR_TOT=VAR_TOT_FROB <- norm(y_obs,"f")
     VAR_GEN=VAR_GEN_FROB <- 0
     VAR_COMPS=VAR_COMPS_FROB <- matrix(0,K,R)
-    VAR_SUPER_COMPS=VAR_SUPER_COMPS_FROB <- matrix(0,q,R)
+    VAR_SUPER_COMPS=VAR_SUPER_COMPS_FROB <- matrix(0,q,R_super)
     VAR_SUPER_COMPS_ALL_Y=VAR_SUPER_COMPS_ALL_Y_FROB <- matrix(0,1,R)
-
+    R_T_super <- ncol(x$mod$T_super)
     t_y_obs <- tcrossprod(y_obs)
 
     T_GEN <- scale(x$mod$T_super)
@@ -558,10 +600,7 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
         t_k_r <- scale(t_r[,k,drop=F],scale=F)
         var_t_k_r_all <- sum(t_k_r^2)
         if(var_t_k_r_all!=0){
-          VAR_COMPS[k,r] <-  get_var_line(t_k_r,scale(y_obs))#(norm(mmultC(t_k_r,b),"f")/VAR_TOT)^2
-          # deno <- sum(diag(t_y_obs))*var_t_k_r_all
-          # numer <- sum(mmultC(y_obs,crossprod(y_obs,t_k_r))*t_k_r)
-          # VAR_COMPS_FROB[k,r] <- numer/deno
+          VAR_COMPS[k,r] <-  get_var_line(t_k_r,scale(y_obs))
           VAR_COMPS_FROB[k,r] <- get_rv(t_k_r,scale(y_obs))
         }
       }
@@ -570,28 +609,23 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
       Y_j <- y_obs[,j,drop=F]
       var_j <- norm(Y_j,"f")
       if(var_j!=0){
-        for(r in 1:R){
+        for(r in 1:R_T_super){
           t_super_r <- x$mod$T_super[,r,drop=F]
           var_t_super_r <- sum(t_super_r^2)
           if(var_t_super_r!=0){
-            VAR_SUPER_COMPS[j,r] <- get_var_line(scale(t_super_r),scale(Y_j))#(norm(mmultC(t_super_r,b),"f")/var_j)^2
-            # t_Y_j <- tcrossprod(Y_j)
-            # t_t_super_r <- tcrossprod(t_super_r)
-            # prod_num <- mmultC(t_Y_j,t_t_super_r)
-            ### OO
+            VAR_SUPER_COMPS[j,r] <- get_var_line(scale(t_super_r),scale(Y_j))
             coef_r <- sum(Y_j*t_super_r)
-            prod_num <- coef_r^2#*tcrossprod(Y_j,t_super_r)
-            ###
+            prod_num <- coef_r^2
             VAR_SUPER_COMPS_FROB[j,r] <- get_rv(scale(t_super_r),scale(Y_j))
           }
         }
       }
     }
-    for(r in 1:R){
+    for(r in 1:R_T_super){
       sc_r <- scale(x$mod$T_super[,r,drop=F],scale=F)
       var_t_super_r <- sum(sc_r^2)
       if(var_t_super_r!=0){
-        VAR_SUPER_COMPS_ALL_Y[r] <- get_var_line(sc_r,scale(y_obs))#(norm(mmultC(sc_r,b),"f")/VAR_TOT)^2
+        VAR_SUPER_COMPS_ALL_Y[r] <- get_var_line(sc_r,scale(y_obs))
         VAR_SUPER_COMPS_ALL_Y_FROB[r] <- get_rv(sc_r,scale(y_obs))
       }
     }
@@ -607,8 +641,10 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
     }
     rownames(VAR_COMPS)=rownames(VAR_COMPS_FROB) <- legend_names_in;
     colnames(VAR_COMPS)=colnames(VAR_COMPS_FROB) <- paste("Comp.",1:R)
-    colnames(VAR_SUPER_COMPS)= colnames(VAR_SUPER_COMPS_ALL_Y) =
-      colnames(VAR_SUPER_COMPS_FROB)= colnames(VAR_SUPER_COMPS_ALL_Y_FROB)<- paste("Super Comp.",1:R)
+    colnames(VAR_SUPER_COMPS) = colnames(VAR_SUPER_COMPS_FROB) <-
+      paste("Super Comp.",1:R_T_super)
+    names(VAR_SUPER_COMPS_ALL_Y) = names(VAR_SUPER_COMPS_ALL_Y_FROB) <-
+      paste("Super Comp.",1:R_T_super)
     rownames(VAR_SUPER_COMPS)=rownames(VAR_SUPER_COMPS_FROB) <- colnames(y_obs)
     return(list(
       Linear=list(VAR_GEN=VAR_GEN,VAR_SUPER_COMPS_ALL_Y=VAR_SUPER_COMPS_ALL_Y,
@@ -617,6 +653,26 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
               VAR_SUPER_COMPS=VAR_SUPER_COMPS_FROB,VAR_COMPS=VAR_COMPS_FROB)))
   }
 
+  get_lambda_from_L0 <- function(Xs,Y,Y_class_dummies=NULL,mode,L0,lambda){
+    if(!is.null(L0)){
+      ps_init <- unlist(lapply(Xs,ncol))
+      sum_ps_init <- sum(ps_init);cum_ps_init <- cumsum(c(0,ps_init))
+      if(mode=="reg"){
+        coco_i <- suppressWarnings(abs(cor(Y,do.call(cbind,Xs),use = "pairwise")))
+        coco_i[which(is.na(coco_i))] <- 0
+        all_maxs_init <- apply(coco_i,2,max)
+      }else{
+        coco_i <- suppressWarnings(abs(cor(Y_class_dummies,do.call(cbind,Xs),use = "pairwise")))
+        coco_i[which(is.na(coco_i))] <- 0
+        all_maxs_init <- apply(coco_i,2,max)
+      }
+      lambda_init <- sort(all_maxs_init,decreasing = T)[min(sum_ps_init,1+L0)]
+    }else{
+      lambda_init <- lambda
+    }
+    lambda_init
+  }
+
   if(lambda<0|lambda>1){
     stop("Choose lambda regularization parameter between 0 and 1",
          call. = FALSE)
@@ -639,7 +695,6 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
   n <- nrow(Y)
   q <- ncol(Y)
   if(keep_imp_mod) model_imputations <- list()
-  has_converged <- maxIter_imput
   id_na <- lapply(Xs,function(x){which(is.na(x[,1]),arr.ind = TRUE)})
   any_na_no_all <- lapply(Xs,function(x){
     oo <- which(is.na(x),arr.ind = TRUE)[,1]
@@ -684,26 +739,13 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
     mu_y <- colMeans(Y_class_dummies)
     sd_y <- sdRcpp(Y_class_dummies)#apply(Y_class_dummies,2,sd)*sqrt((n-1)/n)
   }
+  iter <- 0
   if(length(unlist(id_na))==0){
     ## If there is no missing sample
-    mod <- MddsPLS_core(Xs,Y,lambda=lambda,R=R,mode=mode,L0=L0,verbose=verbose,NZV=NZV)
+    mod <- MddsPLS_core(Xs,Y,lambda=lambda,R=R,mode=mode,L0=L0,mu=mu,deflat=deflat,
+                        weight=weight,NZV=NZV)
   }else{
-    if(!is.null(L0)){
-      ps_init <- unlist(lapply(Xs,ncol))
-      sum_ps_init <- sum(ps_init);cum_ps_init <- cumsum(c(0,ps_init))
-      if(mode=="reg"){
-        coco_i <- suppressWarnings(abs(cor(Y,do.call(cbind,Xs),use = "pairwise")))
-        coco_i[which(is.na(coco_i))] <- 0
-        all_maxs_init <- apply(coco_i,2,max)
-      }else{
-        coco_i <- suppressWarnings(abs(cor(Y_class_dummies,do.call(cbind,Xs),use = "pairwise")))
-        coco_i[which(is.na(coco_i))] <- 0
-        all_maxs_init <- apply(coco_i,2,max)
-      }
-      lambda_init <- sort(all_maxs_init,decreasing = T)[min(sum_ps_init,1+L0)]
-    }else{
-      lambda_init <- lambda
-    }
+    lambda_init <- get_lambda_from_L0(Xs,Y,Y_class_dummies,mode,L0,lambda)
     ## If ther are some missing samples
     for(k in 1:K){## ## Types of imputation for initialization
       if(length(id_na[[k]])>0){
@@ -715,20 +757,18 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
           x_train <- Y[-id_na[[k]],,drop=F]
           x_test <- Y[id_na[[k]],,drop=F]
         }
-        model_init <- mddsPLS(x_train,y_train,R=R,lambda = lambda_init,getVariances=F)
+        model_init <- mddsPLS(x_train,y_train,R=R,lambda = lambda_init,getVariances=F,weight=weight)
         y_test <- predict(model_init,x_test)$y
         Xs[[k]][id_na[[k]],] <- y_test
       }
     }
     if(K>1){
-      mod_0 <- MddsPLS_core(Xs,Y,lambda=lambda,R=R,mode=mode,L0=L0,NZV=NZV)
+      mod_0 <- MddsPLS_core(Xs,Y,lambda=lambda,R=R,mode=mode,L0=L0,mu=mu,deflat=deflat,NZV=NZV,weight=weight)
       if(sum(abs(as.vector(mod_0$S_super)))!=0){
         Mat_na <- matrix(0,n,K)
         for(k in 1:K){
           Mat_na[id_na[[k]],k] <- 1
         }
-        err <- 2
-        iter <- 0
         ## Covariate for imputation is always the same : the projected values of Y on the initial weights
         #S_super_obj <- Y#mod_0$S_super
         if(mode!="reg"){
@@ -742,59 +782,51 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
         # Y_proj <- Y_class_dummies[,-1,drop=F]
         # phi <- tcrossprod(mmultC(Y_proj,solve(crossprod(Y_proj))),Y_proj)
         # ####################
-
-        while(iter<maxIter_imput&err>errMin_imput){
+        V_t_Star <- list()
+        v_t_star_equals <- rep(0,K)
+        while(sum(v_t_star_equals)<K){
           iter <- iter + 1
+          v_t_star_equals <- rep(0,K)
           for(k in 1:K){
             if(length(id_na[[k]])>0){
               no_k <- (1:K)[-k]
               i_k <- id_na[[k]]
-              if(iter>1){
-                # Xs_i <- mod$S_super[-i_k,,drop=FALSE]#_0$S_super[-i_k,,drop=FALSE]
-                # newX_i <- mod$S_super[i_k,,drop=FALSE]#_0$S_super[i_k,,drop=FALSE]
-                Var_selected_k <- which(rowSums(abs(mod$u[[k]]))>NZV)#_0$u[[k]]))!=0)
-              }else{
-                # Xs_i <- mod_0$S_super[-i_k,,drop=FALSE]
-                # newX_i <- mod_0$S_super[i_k,,drop=FALSE]
-                Var_selected_k <- which(rowSums(abs(mod_0$u[[k]]))>NZV)
-              }
-              # for(no_k_i in no_k){
-              #   Var_selected_no_k <- which(rowSums(abs(mod_0$u[[no_k_i]]))>NZV)
-              #   mat_here <- Xs[[no_k_i]][,Var_selected_no_k,drop=F]
-              #   mat_here <- mat_here-mmultC(phi,mat_here)
-              #   S_super_obj <- cbind(S_super_obj,mat_here)
-              # }
-
+              Var_selected_k_previous <- if(iter!=1) V_t_Star[[k]] else NULL
+              u_current_k <- mod_0$u[[k]]#u_current_k <- if(iter==1)mod_0$u[[k]] else mod$u[[k]]
+              Var_selected_k = V_t_Star[[k]] <- which(rowSums(abs(u_current_k))>NZV)
+              v_t_star_equals[k] <- if (setequal(Var_selected_k_previous,Var_selected_k)) 1 else 0
               Xs_i <- S_super_obj[-i_k,,drop=FALSE]
               newX_i <- S_super_obj[i_k,,drop=FALSE]
               Var_selected[k] <- length(Var_selected_k)
-              if(length(Var_selected_k)>0){
-                ## ## ## ## Impute on the selected variables
-                Y_i_k <- Xs[[k]][-i_k,Var_selected_k,drop=FALSE]
-                if(reg_imp_model){
+              if(v_t_star_equals[k]==0){
+                if(length(Var_selected_k)>0){
+                  ## ## ## ## Impute on the selected variables
+                  Y_i_k <- Xs[[k]][-i_k,Var_selected_k,drop=FALSE]
                   ## In that case the value of lambda is computed according to the initial model.
-                  model_here <- MddsPLS_core(Xs_i,Y_i_k,lambda=mod_0$lambda,
-                                             R=R,L0=NULL,NZV=NZV)
-                }else{
-                  ## In that case the value of lambda is taken equal to 0. And so
-                  ## all the variables are taken in the model
                   model_here <- MddsPLS_core(Xs_i,Y_i_k,
-                                             R=R,L0=NULL,NZV=NZV)
-                }
-                mod_i_k <- list(mod=model_here,R=R,mode="reg",maxIter_imput=maxIter_imput)
-                class(mod_i_k) <- "mddsPLS"
-                if(keep_imp_mod){
-                  mod_i_k$Xs <- list(Xs_i)
-                  mod_i_k$Y_0 <- Y_i_k
-                  model_imputations[[k]] <- mod_i_k
-                }
-                Xs[[k]][i_k,Var_selected_k] <- predict.mddsPLS(mod_i_k,newX_i)$y
-              }else{
-                if(keep_imp_mod){
-                  model_imputations[[k]] <- list()
+                                             lambda=lambda_init,
+                                             L0=NULL,
+                                             R=R,mu=mu,deflat=deflat,weight=weight,NZV=NZV)
+                  mod_i_k <- list(mod=model_here,R=R,mode="reg")
+                  class(mod_i_k) <- "mddsPLS"
+                  if(keep_imp_mod){
+                    mod_i_k$Xs <- list(Xs_i)
+                    mod_i_k$Y_0 <- Y_i_k
+                    model_imputations[[k]] <- mod_i_k
+                  }
+                  Xs[[k]][i_k,Var_selected_k] <- predict.mddsPLS(mod_i_k,newX_i)$y
+                  # for(ii_k in i_k){
+                  #   var_not_selected <- c(1:ncol(Xs[[k]]))[-Var_selected_k]
+                  #   Xs[[k]][ii_k,var_not_selected] <- mu_x_s[[k]][var_not_selected]
+                  # }
+                }else{
+                  if(keep_imp_mod){
+                    model_imputations[[k]] <- list()
+                  }
                 }
               }
             }else{
+              v_t_star_equals[k] <- 1
               if(keep_imp_mod){
                 model_imputations[[k]] <- list()
               }
@@ -803,40 +835,10 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
           if(mode!="reg"){
             if(!is.factor(Y))Y <- factor(Y)
           }
-          mod <- MddsPLS_core(Xs,Y,lambda=mod_0$lambda,R=R,
-                              mode=mode,L0=L0,NZV=NZV)#NULL)#######################L0)#
-          if(sum(abs(mod$t_ort))*sum(abs(mod_0$t_ort))!=0){
-            err <- 0
-            # for(r in 1:R){
-            #   n_new <- sqrt(sum(mod$t_ort[,r]^2))
-            #   n_0 <- sqrt(sum(mod_0$t_ort[,r]^2))
-            #   if(n_new*n_0!=0){
-            #     err_i <- abs(1-as.numeric(abs(diag(crossprod(mod$t_ort[,r],
-            #                                                  mod_0$t_ort[,r]))))/(n_new*n_0))
-            #     err <- err + err_i
-            #   }
-            # }
-            tsuper <- mod$T_super
-            covsuper <- crossprod(tsuper)
-            tsuper_0 <- mod_0$T_super
-            covsuper_0 <- crossprod(tsuper_0)
-            mix <- crossprod(tsuper_0,tsuper)
-            numer <- sum(diag(tcrossprod(mix)))
-            denom <- sqrt(sum(diag(mmultC(covsuper_0,covsuper_0)))*
-                            sum(diag(mmultC(covsuper,covsuper))))
-            if(denom>0){
-              err <- 1-numer/denom
-            }
-          }
-          else{
-            err <- 0
-          }
-          if(iter>=maxIter_imput){
-            has_converged <- 0
-          }
-          if(err<errMin_imput){
-            has_converged <- iter
-          }
+          ## lambda_init <- get_lambda_from_L0(Xs,Y,Y_class_dummies,mode,L0,lambda) ## Update lambda or not
+          mod <- MddsPLS_core(Xs,Y,R=R,
+                              lambda=lambda_init,
+                              mode=mode,mu=mu,deflat=deflat,weight=weight,NZV=NZV)
           mod_0 <- mod
         }
         if(keep_imp_mod){
@@ -848,7 +850,6 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
         }
       }
     }
-    mod <- MddsPLS_core(Xs,Y,lambda=lambda,R=R,mode=mode,verbose=verbose,L0=L0,NZV=NZV)
   }
   mod$mu_x_s <- mu_x_s
   mod$sd_x_s <- sd_x_s
@@ -859,20 +860,24 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
     values <- rowSums(mod$u_t_super[[k]])^2
     pos <- which(values>NZV)
     if(length(pos)>0){
+      R_u <- ncol(mod$u[[1]])
+      R_u_super <- ncol(mod$u_t_super[[1]])
       order_values <- order(values[pos],decreasing = T)
       pos_ordered <- pos[order_values]
-      out_k <- matrix(NA,length(pos),2*mod$R)
+      out_k <- matrix(NA,length(pos),R_u+R_u_super)
       coco_Xs_k <- colnames(Xs[[k]])
       if(is.null(coco_Xs_k)){
         rownames(out_k) <- pos_ordered
       }else{
         rownames(out_k) <- coco_Xs_k[pos_ordered]
       }
-      colnames(out_k) <- c(paste("Weights_comp_",1:mod$R,sep=""),
-                           paste("Super_Weights_comp_",1:mod$R,sep=""))
-      for(r in 1:mod$R){
+      colnames(out_k) <- c(paste("Weights_comp_",1:R_u,sep=""),
+                           paste("Super_Weights_comp_",1:R_u_super,sep=""))
+      for(r in 1:R_u){
         out_k[,r] <- mod$u[[k]][pos_ordered,r]
-        out_k[,mod$R+r] <- mod$u_t_super[[k]][pos_ordered,r]
+      }
+      for(r in 1:R_u_super){
+        out_k[,R_u+r] <- mod$u_t_super[[k]][pos_ordered,r]
       }
       var_selected[[k]] <- out_k
     }else{
@@ -887,8 +892,8 @@ mddsPLS <- function(Xs,Y,lambda=0,R=1,mode="reg",L0=NULL,
     if(mode=="regression")names(mod$B) <- names_Xs
     names(mod$Ms) <- names_Xs
   }
-  out <- list(var_selected=var_selected,mod=mod,Xs=Xs,Y_0=Y_0,lambda=lambda,mode=mode,id_na=id_na,
-              maxIter_imput=maxIter_imput,has_converged=has_converged,L0=L0,NZV=NZV)
+  out <- list(var_selected=var_selected,mod=mod,Xs=Xs,Y_0=Y_0,lambda=lambda,mu=mu,mode=mode,id_na=id_na,
+              number_iterations=iter,L0=L0,NZV=NZV)
   class(out) <- "mddsPLS"
   if(keep_imp_mod){
     if(length(names_Xs)!=0) names(model_imputations) <- names_Xs
