@@ -12,6 +12,7 @@
 #' @param alpha.f factor modifying the opacity alpha; typically in [0,1]. Used by \strong{adjustcolor}
 #' @param ylim numeric vectors of length 2, giving the error plot range.
 #' @param no_occurence logical. Whether or not to plot the occurence plot of the \strong{Y} variables. Initialized to \strong{TRUE}.
+#' @param no_plot logical. Whether or not to plot error. Initialized to \strong{FALSE}.
 #' @param main character of \strong{NULL}. If null the title is given to the willing of the software. If \strong{""}, no title is given. Else is what the user wants.
 #' @param ... Other plotting parameters to affect the plot.
 #'
@@ -51,12 +52,14 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
                               ylim=NULL,alpha.f=0.4,
                               no_occurence=T,
                               main=NULL,
+                              no_plot=F,
                               ...){
-  ## Reset personnal plot par() settings
-  opar <- par(no.readonly =TRUE)
-  on.exit(par(opar))
-  ## -----------------------------------
-
+  if(!no_plot){
+    ## Reset personnal plot par() settings
+    opar <- par(no.readonly =TRUE)
+    on.exit(par(opar))
+    ## -----------------------------------
+  }
   res_perf_mdd <- x
   is_L0 <- names(res_perf_mdd[[1]])[2]
   X_all <- scale(do.call(cbind,res_perf_mdd$Xs))
@@ -159,7 +162,7 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
     y_mean <- 1-rowSums(RMSEP[order(RMSEP[,2,drop=FALSE]),3:ncol(RMSEP),drop=FALSE])/sum(TAB)
     main1 <- "Good classification rate versus regularization coefficient mdd-sPLS"
     main2 <- "Occurences per class versus regularization coefficient mdd-sPLS"
-    par(mfrow=c(1,1),ann=T)
+    if(!no_plot)par(mfrow=c(1,1),ann=T)
   }
   # graphics::matplot(sort(RMSEP[,2]),y1,type="l",lwd=4,lty=1,
   #                   ylim=ylim1,col=colors,
@@ -187,37 +190,37 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
     ses <- dat$MSEP + outer(dat$sd, c(1,-1)*delta[jq])
     if(jq==1){
       if(res_perf_mdd$mod=="reg"){
-        with(dat,
-             plot(
-               lambda, MSEP, type="l",xlab=xlab,ylab=ylab1,
-               ylim=ylim,col=colors[jq],lwd=3,
-               main=main1,
-               panel.first=polygon(c(lambda,rev(lambda)), c(ses[,1],rev(ses[,2])),
-                                   border=NA,
-                                   col=adjustcolor(colors[jq],alpha.f = alpha.f))
-             )
+        if(!no_plot)with(dat,
+                         plot(
+                           lambda, MSEP, type="l",xlab=xlab,ylab=ylab1,
+                           ylim=ylim,col=colors[jq],lwd=3,
+                           main=main1,
+                           panel.first=polygon(c(lambda,rev(lambda)), c(ses[,1],rev(ses[,2])),
+                                               border=NA,
+                                               col=adjustcolor(colors[jq],alpha.f = alpha.f))
+                         )
         )
       }else{
-        with(dat,
-             plot(
-               lambda, MSEP, type="l",xlab=xlab,ylab=ylab1,
-               ylim=ylim,col=colors[jq],lwd=3,
-               main=main1,xaxt="n",
-               panel.first=polygon(c(lambda,rev(lambda)), c(ses[,1],rev(ses[,2])),
-                                   border=NA,
-                                   col=adjustcolor(colors[jq],alpha.f = alpha.f))
-             )
+        if(!no_plot)with(dat,
+                         plot(
+                           lambda, MSEP, type="l",xlab=xlab,ylab=ylab1,
+                           ylim=ylim,col=colors[jq],lwd=3,
+                           main=main1,xaxt="n",
+                           panel.first=polygon(c(lambda,rev(lambda)), c(ses[,1],rev(ses[,2])),
+                                               border=NA,
+                                               col=adjustcolor(colors[jq],alpha.f = alpha.f))
+                         )
         )
-        axis(1,at=dat$lambda)
+        if(!no_plot)axis(1,at=dat$lambda)
       }
     }else{
-      with(dat,
-           points(
-             lambda, MSEP, type="l",col=colors[jq],lwd=3,
-             panel.first=polygon(c(lambda,rev(lambda)), c(ses[,1],rev(ses[,2])),
-                                 border=NA,
-                                 col=adjustcolor(colors[jq],alpha.f = alpha.f))
-           )
+      if(!no_plot)with(dat,
+                       points(
+                         lambda, MSEP, type="l",col=colors[jq],lwd=3,
+                         panel.first=polygon(c(lambda,rev(lambda)), c(ses[,1],rev(ses[,2])),
+                                             border=NA,
+                                             col=adjustcolor(colors[jq],alpha.f = alpha.f))
+                       )
       )
     }
     if(delta[jq]==1){
@@ -225,7 +228,7 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
         x <- dat$lambda
         CI.dn <- ses[,2]
         CI.up <- ses[,1]
-        arrows(x,CI.dn,x,CI.up,code=3,length=0.05,angle=90,col=colors[jq])
+        if(!no_plot)arrows(x,CI.dn,x,CI.up,code=3,length=0.05,angle=90,col=colors[jq])
       }
     }
   }
@@ -237,28 +240,32 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
       pos_all <- min(which(y_mean==max(y_mean)))
       pos_one <- min(which(y1==max(y1),arr.ind = T)[,1])
     }
-    abline(v=c(lam_plot[pos_all],
-               lam_plot[pos_one]),lty=4,lwd=2)
+    if(!no_plot)abline(v=c(lam_plot[pos_all],
+                           lam_plot[pos_one]),lty=4,lwd=2)
+    lam_all <- lam_plot[pos_all]
+    lam_one <- lam_plot[pos_one]
   }else{
-    abline(v=c(lam_plot[which(y_mean==min(y_mean))],
-               lam_plot[which(y1==min(y1),arr.ind = T)[,1]]),lty=4,lwd=2)
+    if(!no_plot)abline(v=c(lam_plot[which(y_mean==min(y_mean))],
+                           lam_plot[which(y1==min(y1),arr.ind = T)[,1]]),lty=4,lwd=2)
+    lam_all <- lam_plot[which(y_mean==min(y_mean))]
+    lam_one <- lam_plot[which(y1==min(y1),arr.ind = T)[,1]]
   }
   if(res_perf_mdd$mod!="reg"){
-    points(sort(RMSEP[,2]),y_mean,type = "l",lwd=4,lty=1,
-                     col=adjustcolor(1,alpha.f = 0.2))
-    points(sort(RMSEP[,2]),y_mean,type = "l",lwd=2,lty=3,
-                     col=1)
+    if(!no_plot)points(sort(RMSEP[,2]),y_mean,type = "l",lwd=4,lty=1,
+                       col=adjustcolor(1,alpha.f = 0.2))
+    if(!no_plot)points(sort(RMSEP[,2]),y_mean,type = "l",lwd=2,lty=3,
+                       col=1)
   }
   if(!is.null(legend_names)){
     if(res_perf_mdd$mod!="reg"){
-      legend(pos_legend,
-                       legend = c(paste(legend_names,
-                                        paste(" (",TAB," indiv.)",sep=""),
-                                        sep=""),
-                                  "Mean good classif rate"),
-                       col = c(colors,1),
-                       lty = c(rep(1,length(colors)),3),
-                       lwd=c(rep(2,length(colors),1.5)),bty = "n")
+      if(!no_plot)legend(pos_legend,
+                         legend = c(paste(legend_names,
+                                          paste(" (",TAB," indiv.)",sep=""),
+                                          sep=""),
+                                    "Mean good classif rate"),
+                         col = c(colors,1),
+                         lty = c(rep(1,length(colors)),3),
+                         lwd=c(rep(2,length(colors),1.5)),bty = "n")
     }else{
       if(!is.null(plot_mean)){
         legend_names <- c(legend_names, legend_0)
@@ -270,14 +277,14 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
         lty <- rep(1,length(colors))
         lwd <- rep(2,length(colors))
       }
-      legend(pos_legend,legend = legend_names,
-                       col = col,lty = lty,lwd=lwd,bty = "n")
+      if(!no_plot)legend(pos_legend,legend = legend_names,
+                         col = col,lty = lty,lwd=lwd,bty = "n")
     }
   }
 
   if(plot_mean){
-    points(sort(RMSEP[,2]),y_mean,type="l",
-                     lty=3,lwd=2)
+    if(!no_plot)points(sort(RMSEP[,2]),y_mean,type="l",
+                       lty=3,lwd=2)
     # graphics::points(sort(RMSEP[,2]),y_mean,type="l",
     # col=grDevices::adjustcolor("black",alpha.f = 0.2),
     # lty=1,lwd=4)
@@ -286,13 +293,13 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
   if(is_L0!="L0s"){
     y_card <- card_ranges*diff(range(y1))/diff(range(card_ranges))
     y_card <- y_card - min(y_card) + min(y1)
-    par(new = TRUE)
+    if(!no_plot)par(new = TRUE)
     # graphics::plot(ranges,card_ranges, type = "l", xaxt = "n", yaxt = "n",
     # ylab = "", xlab = "", col = grDevices::adjustcolor("red",0),
     # lty = 1,lwd=5)
-    axis(side = 3,at=ranges,labels=card_ranges, col="red",
-                   col.axis="red")
-    mtext("", side = 3, line = 3, col = "red")
+    if(!no_plot)axis(side = 3,at=ranges,labels=card_ranges, col="red",
+                     col.axis="red")
+    if(!no_plot)mtext("", side = 3, line = 3, col = "red")
   }
   if(res_perf_mdd$mod=="reg"){
     if(is_L0!="L0s"){
@@ -313,11 +320,11 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
       }
     }
     if(!no_occurence){
-      matplot(FREQ[order(FREQ[2]),2],
-                        FREQ[order(FREQ[2]),-c(1:2)]/max(FREQ[order(FREQ[2]),-c(1:2)])*100,
-                        type="l",lwd=4,col=colors,lty=1,
-                        xlab=expression(lambda),ylab=ylab1,
-                        main=main2)
+      if(!no_plot)matplot(FREQ[order(FREQ[2]),2],
+                          FREQ[order(FREQ[2]),-c(1:2)]/max(FREQ[order(FREQ[2]),-c(1:2)])*100,
+                          type="l",lwd=4,col=colors,lty=1,
+                          xlab=expression(lambda),ylab=ylab1,
+                          main=main2)
       if(is_L0!="L0s"){
         pos_y <- unique(seq(1,length(ranges_y),length.out = 15))
         pos_y[length(pos_y)] <- min(max(pos_y),length(ranges_y))
@@ -326,4 +333,7 @@ plot.perf_mddsPLS <- function(x,plot_mean=FALSE,
       }
     }
   }
+  out <- list(optim_para_all = lam_all, optim_para_one = lam_one)
+  class(out) <- "plot.perf_mddsPLS"
+  out
 }
